Imports System.Drawing.Drawing2D

Public Class Form1
    Public Enum AstmClassifications
        None
        ML
        CL
        SP
        SC
    End Enum

    Private Class CoreLayer
        Public Thickness As Single      ' Feet.
        Public Recovery As Single       ' Shown as a percent.
        Public Moisture As String       ' d, m, w, or s.
        Public Plasticity As Single     ' ?
        Public PID As Integer           ' PPM.
        Public FillPicture As Image
        Public AstmClass As AstmClassifications
        Public Description As String

        Public Sub New(ByVal new_Thickness As Single, ByVal new_Recovery As Single, ByVal new_Moisture As String, ByVal new_Plasticity As Single, ByVal new_PID As Integer, ByVal new_fillPicture As Image, ByVal new_AstmClass As AstmClassifications, ByVal new_Description As String)
            Thickness = new_Thickness
            Recovery = new_Recovery
            Moisture = new_Moisture
            Plasticity = new_PLasticity
            PID = new_PID
            FillPicture = new_fillPicture
            AstmClass = new_AstmClass
            Description = new_Description
        End Sub
    End Class

    ' Make and display some data.
    Private Sub btnPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPreview.Click
        ' Define the data.
        DefineData()

        ' Display the preview.
        ppdCoreSample.Width = 800
        ppdCoreSample.Height = 600
        DirectCast(ppdCoreSample, Form).WindowState = FormWindowState.Maximized
        ppdCoreSample.ShowDialog()
    End Sub

    ' Define the data.
    Private m_CoreData() As CoreLayer
    Private Sub DefineData()
        m_CoreData = New CoreLayer() { _
            New CoreLayer(0.5, -1, "d", -1, -1, Nothing, AstmClassifications.None, "Crushed stone and fill."), _
            New CoreLayer(2, -1, "d", -1, 4, imlFillPictures.Images("Silt.bmp"), AstmClassifications.ML, "SILT w/fine sand, not hard, crumbly, dark brown, no odor."), _
            New CoreLayer(1, -1, "m", -1, 4, imlFillPictures.Images("Clay.bmp"), AstmClassifications.CL, "Sandy CLAY, med. orange-brown, no odor."), _
            New CoreLayer(3, -1, "m", -1, 3, imlFillPictures.Images("Sand.bmp"), AstmClassifications.SP, "Fine SAND, same color as above, no odor."), _
            New CoreLayer(4, -1, "m", -1, -1, imlFillPictures.Images("Clay.bmp"), AstmClassifications.CL, "Sandy CLAY, soft, med. orange-gray brown, no odor."), _
            New CoreLayer(4, -1, "s", -1, 3, imlFillPictures.Images("Sand.bmp"), AstmClassifications.SC, "Clayey SAND, gravelly at bottom, no odor."), _
            New CoreLayer(4, -1, "d", -1, 4, imlFillPictures.Images("Silt.bmp"), AstmClassifications.ML, "SILT w/sand, some clay, trace gravel (Till), hard, light yellowish, some brick-red brown coloration at 9.25-10', no odor.") _
        }
    End Sub

    Private Const COL_WID As Integer = 30
    Private Const HEADER_HGT As Integer = 100
    Private Const Y_SCALE As Single = 20    ' Pixels per foot vertically.
    Private Const MAX_FEET As Single = 30

    Private Sub pdocCoreSample_PrintPage(ByVal sender As System.Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles pdocCoreSample.PrintPage
        Dim xmin As Integer = e.MarginBounds.Left
        Dim xmax As Integer = e.MarginBounds.Right
        Dim y As Integer = e.MarginBounds.Top

        ' Draw top area here.
        Using the_font As New Font("Times New Roman", 20, FontStyle.Regular)
            e.Graphics.DrawString("Top stuff goes here...", the_font, Brushes.Black, xmin, y)
            ' ...

            y += 150
        End Using

        ' Draw the core data area.
        Dim ymin As Integer = y
        Using the_font As New Font("Times New Roman", 10, FontStyle.Regular)
            Using center_format As New StringFormat()
                center_format.Alignment = StringAlignment.Center
                center_format.LineAlignment = StringAlignment.Center

                ' Draw column headers.
                Dim x As Integer = xmin + COL_WID \ 2
                y += HEADER_HGT \ 2
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "Depth (feet)")
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "Recovery %")
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "Moisture")
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "Plasticity")
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "PID (ppm)")
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "Graphic Log")
                DrawRotatedString(e.Graphics, the_font, center_format, x, y, "ASTM Class.")

                y += HEADER_HGT \ 2
                e.Graphics.DrawLine(Pens.Black, xmin, y, xmax, y)

                xmin = e.MarginBounds.Left
                Dim y0 As Integer = y + 0.5 * Y_SCALE
                y = y0

                ' Draw the ground line.
                Using ground_pen As New Pen(Color.Black)
                    ground_pen.DashStyle = DashStyle.Custom
                    ground_pen.DashPattern = New Single() {25, 25}
                    e.Graphics.DrawLine(ground_pen, xmin + COL_WID, y0, xmax, y0)
                End Using

                ' Draw the ground line's "0".
                e.Graphics.DrawString("0", the_font, Brushes.Black, xmin + COL_WID \ 2, y, center_format)

                ' Draw sample data.
                Dim total_feet As Single = 0
                For i As Integer = 0 To m_CoreData.Length - 1
                    DrawSample(e.Graphics, m_CoreData(i), xmin, y, xmax, the_font, center_format, total_feet)
                Next i

                ' Draw depth markers.
                For i As Integer = 5 To MAX_FEET - 5 Step 5
                    y = y0 + Y_SCALE * i
                    e.Graphics.DrawLine(Pens.Black, xmin + COL_WID, y, xmin + 2 * COL_WID, y)
                    e.Graphics.DrawString(i.ToString, the_font, Brushes.Black, xmin + COL_WID \ 2, y, center_format)
                Next i

                ' Draw lines and a box around everything.
                y = y0 + MAX_FEET * Y_SCALE
                ' Box.
                e.Graphics.DrawRectangle(Pens.Black, xmin, ymin, xmax - xmin, y - ymin)
                ' Column lines.
                For i As Integer = 1 To 7
                    x = xmin + COL_WID * i
                    e.Graphics.DrawLine(Pens.Black, x, ymin, x, y)
                Next i
            End Using
        End Using

        ' Draw bottom area here.
        y += 20
        Using the_font As New Font("Times New Roman", 20, FontStyle.Regular)
            e.Graphics.DrawString("Bottom stuff goes here...", the_font, Brushes.Black, xmin, y)
            ' ...
        End Using

        e.HasMorePages = False
    End Sub

    ' Draw a string centered at the given position rotated 90 degrees.
    ' Increment x by COL_WID.
    Private Sub DrawRotatedString(ByVal gr As Graphics, ByVal the_font As Font, ByVal center_format As StringFormat, ByRef x As Integer, ByVal y As Integer, ByVal txt As String)
        gr.RotateTransform(-90)
        gr.TranslateTransform(x, y, MatrixOrder.Append)
        gr.DrawString(txt, the_font, Brushes.Black, 0, 0, center_format)

        gr.ResetTransform()
        x += COL_WID
    End Sub

    ' Draw a sample.
    Private Sub DrawSample(ByVal gr As Graphics, ByVal core_data As CoreLayer, ByVal x1 As Integer, ByVal y1 As Single, ByVal x2 As Single, ByVal the_font As Font, ByVal center_format As StringFormat, ByRef total_feet As Single)
        Dim ymin As Integer = y1 + total_feet * Y_SCALE
        Dim ymax As Integer = ymin + core_data.Thickness * Y_SCALE

        ' Divider line below.
        gr.DrawLine(Pens.Black, x1 + COL_WID * 2, ymax, x2, ymax)

        Dim ymid As Integer = ymin + (core_data.Thickness * Y_SCALE) \ 2
        Dim xmid As Integer = x1 + COL_WID + COL_WID \ 2

        ' % Recovery.
        If core_data.Recovery >= 0 Then
            gr.DrawString(core_data.Recovery.ToString(), _
                the_font, Brushes.Black, xmid, ymid, center_format)
        End If

        ' Moisture.
        xmid += COL_WID
        gr.DrawString(core_data.Moisture, _
            the_font, Brushes.Black, xmid, ymid, center_format)

        ' Plasticity.
        xmid += COL_WID
        If core_data.Plasticity >= 0 Then
            gr.DrawString(core_data.Plasticity.ToString(), _
                the_font, Brushes.Black, xmid, ymid, center_format)
        End If

        ' PID.
        xmid += COL_WID
        If core_data.PID >= 0 Then
            gr.DrawString(core_data.PID.ToString(), _
                the_font, Brushes.Black, xmid, ymid, center_format)
        End If

        ' Graphic log.
        xmid += COL_WID
        If core_data.FillPicture IsNot Nothing Then
            Using fill_brush As New TextureBrush(core_data.FillPicture, WrapMode.Tile)
                Dim xmin As Integer = x1 + 5 * COL_WID
                gr.FillRectangle(fill_brush, xmin, ymin, COL_WID, ymax - ymin)
                gr.DrawRectangle(Pens.Black, xmin, ymin, COL_WID, ymax - ymin)
            End Using
        End If

        ' ASTM classification.
        xmid += COL_WID
        If core_data.AstmClass <> AstmClassifications.None Then
            gr.DrawString(core_data.AstmClass.ToString(), _
                the_font, Brushes.Black, xmid, ymid, center_format)
        End If

        ' Description.
        Using descr_format As New StringFormat()
            descr_format.Alignment = StringAlignment.Near
            descr_format.LineAlignment = StringAlignment.Center

            Dim x As Integer = x1 + 7 * COL_WID + 4
            xmid += COL_WID
            Dim rect As New Rectangle(x, ymin, x2 - x - 8, ymax - ymin)
            gr.DrawString(core_data.Description, _
                the_font, Brushes.Black, rect, descr_format)
        End Using

        total_feet += core_data.Thickness
    End Sub
End Class
